
set search_path to %%_util;


CREATE TABLE %%_util.access_role (
id serial primary key,
rolename varchar not null
);

CREATE TABLE %%_util.access_user (
id serial primary key,
username varchar not null
);

CREATE UNIQUE INDEX access_role_rolename_idx ON %%_util.access_role(rolename);
CREATE UNIQUE INDEX access_user_username_idx ON %%_util.access_user(username);


CREATE TABLE %%_util.node_to_role (
id serial primary key,
node_id bigint not null references %%_util.tree_node(id) on delete cascade,
role_id serial references %%_util.access_role(id) on delete cascade
);


CREATE TABLE %%_util.user_to_role (
id serial primary key,
tree_id integer not null references %%_util.tree(id) on delete cascade,
user_id serial references %%_util.access_user(id) on delete cascade,
role_id serial references %%_util.access_role(id) on delete cascade
);


CREATE UNIQUE INDEX user_to_role_idx ON %%_util.user_to_role(tree_id,user_id,role_id);
CREATE UNIQUE INDEX node_to_role_idx ON %%_util.node_to_role(node_id,role_id);







CREATE TYPE %%.tree_node_with_role AS (
	id bigint,
	node_name varchar,
	node_data_id bigint,
    weight double precision,
    internal_weight bigint,
    level integer,
    parent_id bigint,
	rolename varchar
);


CREATE TYPE %%.user_with_role AS (
	username varchar,
	rolename varchar
);



CREATE SCHEMA %2%_util;
CREATE SCHEMA %2%;

CREATE VIEW %2%_util.tree_node AS 
SELECT  distinct n.*
FROM %%_util.tree_node n 
left JOIN %%_util.node_to_role nr ON (n.id = nr.node_id)
JOIN %%_util.user_to_role ur ON (ur.tree_id = n.tree_id)
JOIN %%_util.access_user u ON (u.id = ur.user_id)	
WHERE ((nr.role_id = ur.role_id)  or nr.role_id is null) and u.username = CURRENT_USER
;

CREATE VIEW %2%_util.tree_node_metadata AS 
SELECT  distinct m.*
FROM %%_util.tree_node n
JOIN %%_util.tree_node_metadata m ON (n.id = m.id) 
left JOIN %%_util.node_to_role nr ON (n.id = nr.node_id)
JOIN %%_util.user_to_role ur ON (ur.tree_id = n.tree_id)
JOIN %%_util.access_user u ON (u.id = ur.user_id)	
WHERE ((nr.role_id = ur.role_id)  or nr.role_id is null) and u.username = CURRENT_USER
;


CREATE VIEW %2%_util.tree AS 
SELECT distinct t.*
FROM %%_util.tree t
JOIN %%_util.user_to_role ur ON (t.id = ur.tree_id)
JOIN %%_util.access_user u ON (u.id = ur.user_id)
WHERE u.username = CURRENT_USER
;









GRANT USAGE ON SCHEMA %2%_util TO PUBLIC;
GRANT USAGE ON SCHEMA %2% TO PUBLIC;
GRANT SELECT ON %2%_util.tree_node_metadata TO PUBLIC;
GRANT SELECT ON %2%_util.tree_node TO PUBLIC;
GRANT SELECT ON %2%_util.tree TO PUBLIC;
